
/**
 * Write a description of class Palindrome here.
 *
 * @author Pete Lemieszewski
 * @version 1.0
 */
import java.lang.Exception;
import java.util.NoSuchElementException;
public class Palindrome
{

    private QueueMyLinkedList q;
    private StackMyLinkedList stk;

    /**
     * Constructor for objects of class Palindrome
     */
    public Palindrome()
    {
        q = new QueueMyLinkedList();
        stk = new StackMyLinkedList();

    }

    /**
     * Cleanup the queue and stack in order to reuse them.
     * Not strictly needed as we normally dequeue() and pop() until empty anyway.
     * We want to defensively guard against some unexpected, unlikely error.
     * Causes the queue and stack to be empty, and should properly be thought of as
     * an ADT specification for stacks and queues... even if no errors occur, there
     * may be a valid reason to empty a stack, or a queue. What might that reason be?
     */
    public void makeEmpty()
    {
        int size = q.getSize();
        if (size != 0) {
            while (size != 0) {
                q.dequeue();
                size--;
            }
        }
        size = stk.getSize();
        if (size != 0) {
            while (size != 0) {
                stk.pop();
                size--;
            }
        }        
    }
    /**
     * The public isPalindrome method will determine if a string of integers represents
     * a palindrome... or not!
     *
     * @param  numbers: a character string represent integer numbers
     * @return returns either true or false
     * @throws PalindromeException in case of a stack or queue implementation error
     */
    public boolean isPalindrome(String numbers) throws PalindromeException
    {
        Integer value;
        makeEmpty();  // just in case!
        for(int posn = 0; posn < numbers.length(); posn++) {
            String str = numbers.substring(posn, posn + 1);
            try {
                value = Integer.valueOf(str);
            } catch (NumberFormatException e) {
                return false;
            }
            //
            // Even though we parsed the input string and verified we have an Integer,
            // we'll write "defensive code" in an attempt to catch any exceptions
            // that our ADT specifications state can be thrown, and "defend against"
            // an unexpected abend.
            //
            try {
                stk.push(value);
                q.enqueue(value);
            }
            catch (NullPointerException e) {
                throw new PalindromeException("Not sure how we got a null string, but we did.");
            }
        }
        boolean compareStackAndQueue;
        try {
            compareStackAndQueue = isPalindrome(numbers.length());
        } catch (PalindromeException p) {
            throw p;        
        }
        return compareStackAndQueue;
    }

    /**
     * helper function for the public version of isPalindrome
     *
     * @param  length, the size of the string
     * @return true if the items taken off the stack and queue match
     * @throws PalindromeException if the stack or queue is (unexpectedly) empty.
     */
    private boolean isPalindrome(int length) throws PalindromeException
    {
        int i = 0;          // useful for debuging
        boolean tis = true;
        Integer stackValue; // we could use pod type of int here too.
        Integer queueValue; // ditto
        while(i < length){
            //
            // We don't expect to come across an empty queue or an empty stack. However,
            // we write defensive code to guard against possible errors in our implementations.
            //
            try {
                stackValue = (int)stk.pop();
                queueValue = (int)q.dequeue();               
            } catch (NoSuchElementException e) {
                throw new PalindromeException("The stack and/or queue is empty! ");
            }
            i++;
            if(stackValue.equals(queueValue)) {
                continue;
            }
            else {                
                tis = false;
            }

        }

        return tis;
    } 

    public void display(String s, boolean tis)
    {
        System.out.printf("isPalindrome() says string %s is...\n", s);
        if (tis)
            System.out.println("a palindrome. \n");
        else 
            System.out.println("NOT a palindrome. \n");
    }

    /**
     * This method tests the queue and stack implementations via
     * the Palindrome class.
     */
    public static void main(String[] args){
        Palindrome p = new Palindrome();
        String [] s = {"123454321", "1234567890", "1","abc"};
        //answers for s:IS,          ISN'T,        IS, ISN'T
        boolean b;        
        for (int i=0; i <s.length; i++) {
            //
            // Since isPalindrome can throw an exception we need to try and catch it.            
            //
            try {
            b = p.isPalindrome(s[i]);
            p.display(s[i], b);                     
            } catch (PalindromeException e) {              
              System.out.printf("String %s caused an exception: %s. \n", s[i], e);              
            }
        }
    }
}
